/* -*- mode: c++ -*- */
/**
 * @file   Transforms.cc
 * @author Sebastien Fourey <http://www.greyc.ensicaen.fr/~seb>
 * @date   Sat Aug 18 2007
 * 
 * @brief  
 * @copyright
 * Copyright or  or Copr. Sbastien Fourey
 * 
 * Sbastien.Fourey - http://www.greyc.ensicaen.fr/~seb
 * 
 * 
 * This source code is part of the Board project, a C++ library whose
 * purpose is to allow simple drawings in EPS, FIG or SVG files.
 * 
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms
 * 
 */

#include "board/Rect.h"
#include "board/Shapes.h"
#include "board/Transforms.h"
#include <cmath>

namespace BoardLib {

//
// Transform
// 

float
Transform::rounded( float x ) const
{
  return round( 10000*x ) / 10000;
} 

float
Transform::mapX( float x ) const
{
  return rounded( x * _scale + _deltaX );
}

float
Transform::scale( float x ) const
{
  return rounded( x * _scale );
}

void
Transform::apply( float & x, float & y ) const
{
  x = mapX( x );
  y = mapY( y );
}

void
Transform::furtherScale( float x )
{
  _scale *= x;
}

//
// TransformEPS
// 

float
TransformEPS::mapY( float y ) const
{
  return rounded( y * _scale + _deltaY );
}

void
TransformEPS::setBoundingBox( const Rect & rect )
{
  float ppmm = (720/254.0f);  
  if ( rect.height/rect.width > 27.7/19.0 ) {
    _scale = 277 * ppmm / rect.height;
  } else {
    _scale = 190 * ppmm / rect.width;
  }
  _deltaX = 0.5 * 210 * ppmm - _scale * ( rect.left + 0.5 * rect.width );
  _deltaY = 0.5 * 297 * ppmm - _scale * ( rect.top - 0.5 * rect.height );  
}

//
// TransformFIG
// 

float
TransformFIG::rounded( float x ) const
{
  return round( x );
}

float
TransformFIG::mapY( float y ) const
{
  // float ppmm = 1200/25.4;
  float ppmm = 1143/25.4f;
  return rounded( ( 297*ppmm ) - ( y * _scale + _deltaY ) );
}

int
TransformFIG::mapWidth( float width ) const
{
  // FIG width unit is 1/160 inch
  // Postscript points are 1/72 inch
  int result =  static_cast<int>( round( 160 * (width / 72.0 ) ) );
  return result?result:1;
}

void
TransformFIG::setBoundingBox( const Rect & rect )
{
  // float ppmm = (1200/25.4);
  float ppmm = (1143/25.4f);
  if ( rect.height/rect.width > 27.7/19.0 ) {
    _scale = 277 * ppmm / rect.height;
  } else {
    _scale = ( 190 * ppmm ) / rect.width;
  }
  _deltaX = 0.5 * 210 * ppmm - _scale * ( rect.left + 0.5 * rect.width );
  _deltaY = 0.5 * 297 * ppmm - _scale * ( rect.top - 0.5 * rect.height );  
}

void
TransformFIG::setDepthRange( const std::vector<Shape*> & shapes )
{
  maxDepth = 0;
  minDepth = std::numeric_limits<unsigned int>::max();
  std::vector<Shape*>::const_iterator i = shapes.begin();
  std::vector<Shape*>::const_iterator end = shapes.end();
  while ( i != end ) {
    if ( (*i)->depth > maxDepth ) maxDepth = (*i)->depth;
    if ( (*i)->depth < minDepth ) minDepth = (*i)->depth;
    ++i;
  }
}

unsigned int
TransformFIG::mapDepth( unsigned int depth ) const
{
  if ( maxDepth - minDepth > 998 ) {
    double range = maxDepth - minDepth;
    return static_cast<unsigned int>( 1 + round( ( ( depth - minDepth ) / range ) * 998 ) );
  } else {
    return 1 + depth - minDepth;
  }
}

//
// TransformSVG
// 

float
TransformSVG::rounded( float x ) const
{
  return round( 100*x ) / 100.0f;
} 

float
TransformSVG::mapY( float y ) const
{
  const float ppmm = 720/254.0f;
  return rounded( ( 297 * ppmm ) - ( y * _scale + _deltaY ) );
}

float
TransformSVG::mapWidth( float width ) const
{
  const float ppmm = 72.0/25.4f;
  return round( 1000 * width / ppmm ) / 1000.0;
}

void
TransformSVG::setBoundingBox( const Rect & rect )
{
  const float ppmm = 720/254.0f;
  if ( rect.height/rect.width > 27.7/19.0 ) {
    _scale = 277 * ppmm / rect.height;
  } else {
    _scale = 190 * ppmm / rect.width;
  }
  _deltaX = 0.5 * 210 * ppmm - _scale * ( rect.left + 0.5 * rect.width );
  _deltaY = 0.5 * 297 * ppmm - _scale * ( rect.top - 0.5 * rect.height );
}

} // namespace BoardLib
